/*
 ******************************************************************************
 *
 *	ParcPlace Systems Inc.
 *	Copyright (c) 1992, ParcPlace Systems.	USA	All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	resname.H
 *
 *	Functions(s):
 *
 *	Description:
 *		Resource names
 *
 *	RCSid = "$Id: resname.H,v 4.34.1.1 1993/06/01 22:56:24 mth Exp $"
 *
 ****************************************************************************
 */
#ifndef OI_resname_H
#define OI_resname_H

/*
* $XConsortium: StringDefs.h,v 1.49 89/12/12 19:24:02 swick Exp $
* $oHeader: StringDefs.h,v 1.3 88/08/19 16:39:44 asente Exp $
*/

/***********************************************************
Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts,
and the Massachusetts Institute of Technology, Cambridge, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the names of Digital or MIT not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

******************************************************************/

	extern	const	char	OI_DefaultFont[];

	extern	const	char	OI_default_background[];
	extern	const	char	OI_default_font[];
	extern	const	char	OI_default_foreground[];
	extern	const	char	OI_default_language[];

	extern	const	char	OI_c_AbbrCellPaintAsCell[];
	extern	const	char	OI_c_AcceleratorLabel[];
	extern	const	char	OI_c_Accelerators[];
	extern	const	char	OI_c_ActiveFile[];
	extern	const	char	OI_c_Alignment[];
	extern	const	char	OI_c_AppHelpFile[];
	extern	const	char	OI_c_AppHelpTopic[];
	extern	const	char	OI_c_Application[];
	extern	const	char	OI_c_Argp[];
	extern	const	char	OI_c_Augment[];
	extern	const	char	OI_c_AutoCntrlrVisibility[];
	extern	const	char	OI_c_AutoResize[];
	extern	const	char	OI_c_BackgroundPixmap[];
	extern	const	char	OI_c_Background[];
	extern	const	char	OI_c_BarActiveColor[];
	extern	const	char	OI_c_BarInactiveColor[];
	extern	const	char	OI_c_BarSize[];
	extern	const	char	OI_c_BaseClass[];
	extern	const	char	OI_c_BevelStyle[];
	extern	const	char	OI_c_BevelWidth[];
	extern	const	char	OI_c_BitmapFilePath[];
	extern	const	char	OI_c_Bitmap[];
	extern	const	char	OI_c_BorderColor[];
	extern	const	char	OI_c_BorderWidth[];
	extern	const	char	OI_c_BottomBevelColor[];
	extern	const	char	OI_c_BottomSpace[];
	extern	const	char	OI_c_Callback[];
	extern	const	char	OI_c_Callbacks[];
	extern	const	char	OI_c_CellOutline[];
	extern	const	char	OI_c_CharEncodeType[];
	extern	const	char	OI_c_Children[];
	extern	const	char	OI_c_ClickDelta[];
	extern	const	char	OI_c_ClickSelect[];
	extern	const	char	OI_c_Clip[];
	extern	const	char	OI_c_Collapse[];
	extern	const	char	OI_c_Column[];
	extern	const	char	OI_c_Config[];
	extern	const	char	OI_c_Controllers[];
	extern	const	char	OI_c_ControllerVisibility[];
	extern	const	char	OI_c_Cursor[];
	extern	const	char	OI_c_CutPaste[];
	extern	const	char	OI_c_DecimalPosition[];
	extern	const	char	OI_c_DefaultCell[];
	extern	const	char	OI_c_DefaultHorizontalSpace[];
	extern	const	char	OI_c_DefaultKey[];
	extern	const	char	OI_c_DefaultLanguage[];
	extern	const	char	OI_c_DefaultText[];
	extern	const	char	OI_c_DefaultVerticalSpace[];
	extern	const	char	OI_c_Defaults[];
	extern	const	char	OI_c_Directory[];
	extern	const	char	OI_c_DisplayLength[];
	extern	const	char	OI_c_DownColor[];
	extern	const	char	OI_c_DownIsBackground[];
	extern	const	char	OI_c_DragRightDistance[];
	extern	const	char	OI_c_DynamicResources[];
	extern	const	char	OI_c_EndType[];
	extern	const	char	OI_c_FastDraw[];
	extern	const	char	OI_c_File[];
	extern	const	char	OI_c_FilterLabel[];
	extern	const	char	OI_c_Filter[];
	extern	const	char	OI_c_FocusFrameColor[];
	extern	const	char	OI_c_FocusFramePixmap[];
	extern	const	char	OI_c_FocusFrameWidth[];
	extern	const	char	OI_c_FocusIndicator[];
	extern	const	char	OI_c_FocusPolicy[];
	extern	const	char	OI_c_FontSet[];
	extern	const	char	OI_c_Font[];
	extern	const	char	OI_c_Foreground[];
	extern	const	char	OI_c_FrameWidth[];
	extern	const	char	OI_c_Function[];
	extern	const	char	OI_c_Geometry[];
	extern	const	char	OI_c_Gravity[];
	extern	const	char	OI_c_HandleIndent[];
	extern	const	char	OI_c_Height[];
	extern	const	char	OI_c_HelpFile[];
	extern	const	char	OI_c_HelpTopic[];
	extern	const	char	OI_c_HelpTranslations[];
	extern	const	char	OI_c_Highlight[];
	extern	const	char	OI_c_HorzMove[];
	extern	const	char	OI_c_HorzResize[];
	extern	const	char	OI_c_IconFilename[];
	extern	const	char	OI_c_IconGeometry[];
	extern	const	char	OI_c_IconX[];
	extern	const	char	OI_c_IconY[];
	extern	const	char	OI_c_Iconic[];
	extern	const	char	OI_c_IgnoreMouseLock[];
	extern	const	char	OI_c_IgnoreModifierMask[];
	extern	const	char	OI_c_InactiveCursor[];
	extern	const	char	OI_c_IncDecChk[];
	extern	const	char	OI_c_IndependentLoc[];
	extern	const	char	OI_c_InlineEnhance[];
	extern	const	char	OI_c_Interval[];
	extern	const	char	OI_c_KeyboardInput[];
	extern	const	char	OI_c_Label[];
	extern	const	char	OI_c_LanguageServerFocusinAction[];
	extern	const	char	OI_c_LanguageServer[];
	extern	const	char	OI_c_Language[];
	extern	const	char	OI_c_Layout[];
	extern	const	char	OI_c_LeftSpace[];
	extern	const	char	OI_c_Length[];
	extern	const	char	OI_c_Long[];
	extern	const	char	OI_c_Longterm[];
	extern	const	char	OI_c_MainMenu[];
	extern	const	char	OI_c_MapRaised[];
	extern	const	char	OI_c_Margin[];
	extern	const	char	OI_c_MappingDelay[];
	extern	const	char	OI_c_Mask[];
	extern	const	char	OI_c_MaxChars[];
	extern	const	char	OI_c_MaxLabel[];
	extern	const	char	OI_c_MaxLines[];
	extern	const	char	OI_c_MaxSize[];
	extern	const	char	OI_c_MaximumLength[];
	extern	const	char	OI_c_Maximum[];
	extern	const	char	OI_c_MinLabel[];
	extern	const	char	OI_c_MinSize[];
	extern	const	char	OI_c_Minimum[];
	extern	const	char	OI_c_MnemonicLabel[];
	extern	const	char	OI_c_MnemonicStyle[];
	extern	const	char	OI_c_Mnemonics[];
	extern	const	char	OI_c_Model[];
	extern	const	char	OI_c_Modeless[];
	extern	const	char	OI_c_ModelSpecificHints[];
	extern	const	char	OI_c_MotifPushpin[];
	extern	const	char	OI_c_MotionCallback[];
	extern	const	char	OI_c_Move[];
	extern	const	char	OI_c_NameObjects[];
	extern	const	char	OI_c_NoInherit[];
	extern	const	char	OI_c_NumTick[];
	extern	const	char	OI_c_ObjectBoxHeight[];
	extern	const	char	OI_c_ObjectBoxWidth[];
	extern	const	char	OI_c_ObjectResources[];
	extern	const	char	OI_c_Object[];
	extern	const	char	OI_c_Orientation[];
	extern	const	char	OI_c_Override[];
	extern	const	char	OI_c_PasteAtPointer[];
	extern	const	char	OI_c_Path[];
	extern	const	char	OI_c_Persistent[];
	extern	const	char	OI_c_Pinned[];
	extern	const	char	OI_c_PixelRange[];
	extern	const	char	OI_c_PixelsX[];
	extern	const	char	OI_c_PixelsY[];
	extern	const	char	OI_c_Placement[];
	extern	const	char	OI_c_Position[];
	extern	const	char	OI_c_Pushpin[];
	extern	const	char	OI_c_RealTimeDrag[];
	extern	const	char	OI_c_ResizeByGrip[];
	extern	const	char	OI_c_Resize[];
	extern	const	char	OI_c_ResourceFile[];
	extern	const	char	OI_c_RestrictTerm[];
	extern	const	char	OI_c_ReverseVideo[];
	extern	const	char	OI_c_RightMargin[];
	extern	const	char	OI_c_RightSpace[];
	extern	const	char	OI_c_Row[];
	extern	const	char	OI_c_ScrollMenuTimeout[];
	extern	const	char	OI_c_Selected[];
	extern	const	char	OI_c_Separator[];
	extern	const	char	OI_c_ShapeFile[];
	extern	const	char	OI_c_ShowSep[];
	extern	const	char	OI_c_ShowValue[];
	extern	const	char	OI_c_SizeTrack[];
	extern	const	char	OI_c_Space[];
	extern	const	char	OI_c_SpanUpdate[];
	extern	const	char	OI_c_SpanX[];
	extern	const	char	OI_c_SpanY[];
	extern	const	char	OI_c_StandardColormap[];
	extern	const	char	OI_c_State[];
	extern	const	char	OI_c_SubMenuType[];
	extern	const	char	OI_c_Subbox[];
	extern	const	char	OI_c_SubclassFilePath[];
	extern	const	char	OI_c_Submenu[];
	extern	const	char	OI_c_Substitute[];
	extern	const	char	OI_c_Synchronous[];
	extern	const	char	OI_c_TabWidth[];
	extern	const	char	OI_c_TextFile[];
	extern	const	char	OI_c_Text[];
	extern	const	char	OI_c_TickLabels[];
	extern	const	char	OI_c_TickType[];
	extern	const	char	OI_c_Title[];
	extern	const	char	OI_c_TopBevelColor[];
	extern	const	char	OI_c_TopLevel[];
	extern	const	char	OI_c_TopSpace[];
	extern	const	char	OI_c_Track[];
	extern	const	char	OI_c_TrackCells[];
	extern	const	char	OI_c_Translations[];
	extern	const	char	OI_c_Underline[];
	extern	const	char	OI_c_UnitX[];
	extern	const	char	OI_c_UnitY[];
	extern	const	char	OI_c_Unselect[];
	extern	const	char	OI_c_UserArgRqd[];
	extern	const	char	OI_c_UserArg[];
	extern	const	char	OI_c_UserVariable[];
	extern	const	char	OI_c_ValidateText[];
	extern	const	char	OI_c_Value[];
	extern	const	char	OI_c_VertMove[];
	extern	const	char	OI_c_VertResize[];
	extern	const	char	OI_c_ViewChars[];
	extern	const	char	OI_c_ViewHeight[];
	extern	const	char	OI_c_ViewLines[];
	extern	const	char	OI_c_ViewSize[];
	extern	const	char	OI_c_ViewWidth[];
	extern	const	char	OI_c_ViewX[];
	extern	const	char	OI_c_ViewY[];
	extern	const	char	OI_c_ViewportFitsText[];
	extern	const	char	OI_c_Width[];
	extern	const	char	OI_c_WmIgnoresPPosition[];
	extern	const	char	OI_c_Wrap[];
	extern	const	char	OI_c_WrapAround[];
	extern	const	char	OI_c_WrapLimit[];
	extern	const	char	OI_c_X[];
	extern	const	char	OI_c_Y[];
	extern	const	char	OI_n_abbrCellPaintAsCell[];
	extern	const	char	OI_n_acceleratorLabel[];
	extern	const	char	OI_n_accelerators[];
	extern	const	char	OI_n_activeFile[];
	extern	const	char	OI_n_alignment[];
	extern	const	char	OI_n_appHelpFile[];
	extern	const	char	OI_n_appHelpTopic[];
	extern	const	char	OI_n_application[];
	extern	const	char	OI_n_argp[];
	extern	const	char	OI_n_augment[];
	extern	const	char	OI_n_autoCntrlrVisibility[];
	extern	const	char	OI_n_autoResize[];
	extern	const	char	OI_n_backgroundPixmap[];
	extern	const	char	OI_n_background[];
	extern	const	char	OI_n_barActiveColor[];
	extern	const	char	OI_n_barInactiveColor[];
	extern	const	char	OI_n_barSize[];
	extern	const	char	OI_n_baseClass[];
	extern	const	char	OI_n_bevelStyle[];
	extern	const	char	OI_n_bevelWidth[];
	extern	const	char	OI_n_bitmapFilePath[];
	extern	const	char	OI_n_bitmap[];
	extern	const	char	OI_n_borderColor[];
	extern	const	char	OI_n_borderWidth[];
	extern	const	char	OI_n_border[];
	extern	const	char	OI_n_bottomBevelColor[];
	extern	const	char	OI_n_bottomSpace[];
	extern	const	char	OI_n_callback[];
	extern	const	char	OI_n_callbacks[];
	extern	const	char	OI_n_cbAnimate[];
	extern	const	char	OI_n_cbCellAction[];
	extern	const	char	OI_n_cbClick[];
	extern	const	char	OI_n_cbCmdEntryChk[];
	extern	const	char	OI_n_cbCtlr1d[];
	extern	const	char	OI_n_cbCtlr2d[];
	extern	const	char	OI_n_cbDecCheck[];	// OBSOLETE, use OI_n_cbSeqDecCheck
	extern	const	char	OI_n_cbDeiconify[];
	extern	const	char	OI_n_cbDestroy[];
	extern	const	char	OI_n_cbDragPreview[];
	extern	const	char	OI_n_cbDropPreview[];
	extern	const	char	OI_n_cbDrop[];
	extern	const	char	OI_n_cbEfCharChg[];
	extern	const	char	OI_n_cbEfCharCheck[];
	extern	const	char	OI_n_cbEfCharChk[];	// OBSOLETE, use OI_n_cbEfCharCheck
	extern	const	char	OI_n_cbEfEntryCheck[];
	extern	const	char	OI_n_cbEfEntryChk[];	// OBSOLETE, use OI_n_cbEfEntryrCheck
	extern	const	char	OI_n_cbExpose[];
	extern	const	char	OI_n_cbFocusin[];
	extern	const	char	OI_n_cbFocusout[];
	extern	const	char	OI_n_cbGripMoved[];
	extern	const	char	OI_n_cbGripStart[];
	extern	const	char	OI_n_cbGripStop[];
	extern	const	char	OI_n_cbIconify[];
	extern	const	char	OI_n_cbIncCheck[];	// OBSOLETE, use OI_n_cbSeqIncCheck
	extern	const	char	OI_n_cbKeyHelp[];
	extern	const	char	OI_n_cbMenuEnd[];
	extern	const	char	OI_n_cbMtCharChg[];
	extern	const	char	OI_n_cbMtCharCheck[];
	extern	const	char	OI_n_cbMtCharChk[];	// OBSOLETE, use OI_n_cbMtCharCheck
	extern	const	char	OI_n_cbMtEntryCheck[];
	extern	const	char	OI_n_cbMtEntryChk[];	// OBSOLETE, use OI_n_cbMtEntryCheck
	extern	const	char	OI_n_cbOIDestroy[];
	extern	const	char	OI_n_cbOIResize[];
	extern	const	char	OI_n_cbOITrackReloc[];
	extern	const	char	OI_n_cbPanPaint[];
	extern	const	char	OI_n_cbPin[];
	extern	const	char	OI_n_cbPrePopup[];
	extern	const	char	OI_n_cbProtocolDelete[];
	extern	const	char	OI_n_cbResize[];
	extern	const	char	OI_n_cbSaveYourself[];
	extern	const	char	OI_n_cbSelectionConvert[];
	extern	const	char	OI_n_cbSeqDecCheck[];
	extern	const	char	OI_n_cbSeqIncCheck[];
	extern	const	char	OI_n_cbShutdown[];
	extern	const	char	OI_n_cbTermAction[];
	extern	const	char	OI_n_cbUnpin[];
	extern	const	char	OI_n_cellOutline[];
	extern	const	char	OI_n_charEncodeType[];
	extern	const	char	OI_n_children[];
	extern	const	char	OI_n_clickDelta[];
	extern	const	char	OI_n_clickSelect[];
	extern	const	char	OI_n_clip[];
	extern	const	char	OI_n_collapse[];
	extern	const	char	OI_n_colormap[];
	extern	const	char	OI_n_column[];
	extern	const	char	OI_n_config[];
	extern	const	char	OI_n_controllers[];
	extern	const	char	OI_n_controllerVisibility[];
	extern	const	char	OI_n_cursor[];
	extern	const	char	OI_n_cutPaste[];
	extern	const	char	OI_n_data[];
	extern	const	char	OI_n_decimalPosition[];
	extern	const	char	OI_n_defaultCell[];
	extern	const	char	OI_n_defaultHorizontalSpace[];
	extern	const	char	OI_n_defaultKey[];
	extern	const	char	OI_n_defaultLanguage[];
	extern	const	char	OI_n_defaultText[];
	extern	const	char	OI_n_defaultVerticalSpace[];
	extern	const	char	OI_n_defaults[];
	extern	const	char	OI_n_directory[];
	extern	const	char	OI_n_displayLength[];
	extern	const	char	OI_n_downColor[];
	extern	const	char	OI_n_downIsBackground[];
	extern	const	char	OI_n_dragRightDistance[];
	extern	const	char	OI_n_dynamicResources[];
	extern	const	char	OI_n_endType[];
	extern	const	char	OI_n_fastDraw[];
	extern	const	char	OI_n_file[];
	extern	const	char	OI_n_filterLabel[];
	extern	const	char	OI_n_filter[];
	extern	const	char	OI_n_focusFrameColor[];
	extern	const	char	OI_n_focusFramePixmap[];
	extern	const	char	OI_n_focusFrameWidth[];
	extern	const	char	OI_n_focusIndicator[];
	extern	const	char	OI_n_focusPolicy[];
	extern	const	char	OI_n_fontSet[];
	extern	const	char	OI_n_font[];
	extern	const	char	OI_n_foreground[];
	extern	const	char	OI_n_frameWidth[];
	extern	const	char	OI_n_function[];
	extern	const	char	OI_n_geometry[];
	extern	const	char	OI_n_gravity[];
	extern	const	char	OI_n_handleIndent[];
	extern	const	char	OI_n_height[];
	extern	const	char	OI_n_helpFile[];
	extern	const	char	OI_n_helpTopic[];
	extern	const	char	OI_n_helpTranslations[];
	extern	const	char	OI_n_highlight[];
	extern	const	char	OI_n_horzMove[];
	extern	const	char	OI_n_horzResize[];
	extern	const	char	OI_n_iconFilename[];
	extern	const	char	OI_n_iconGeometry[];
	extern	const	char	OI_n_iconX[];
	extern	const	char	OI_n_iconY[];
	extern	const	char	OI_n_iconic[];
	extern	const	char	OI_n_ignoreMouseLock[];
	extern	const	char	OI_n_ignoreModifierMask[];
	extern	const	char	OI_n_inactiveCursor[];
	extern	const	char	OI_n_incDecChk[];
	extern	const	char	OI_n_independentLoc[];
	extern	const	char	OI_n_inlineEnhance[];
	extern	const	char	OI_n_interval[];
	extern	const	char	OI_n_keyboardInput[];
	extern	const	char	OI_n_label[];
	extern	const	char	OI_n_languageServerFocusinAction[];
	extern	const	char	OI_n_languageServer[];
	extern	const	char	OI_n_language[];
	extern	const	char	OI_n_layout[];
	extern	const	char	OI_n_leftSpace[];
	extern	const	char	OI_n_length[];
	extern	const	char	OI_n_long[];
	extern	const	char	OI_n_longterm[];
	extern	const	char	OI_n_mainMenu[];
	extern	const	char	OI_n_mapRaised[];
	extern	const	char	OI_n_mappedWhenManaged[];
	extern	const	char	OI_n_mappingDelay[];
	extern	const	char	OI_n_marginHeight[];
	extern	const	char	OI_n_marginWidth[];
	extern	const	char	OI_n_mask[];
	extern	const	char	OI_n_maxChars[];
	extern	const	char	OI_n_maxLabel[];
	extern	const	char	OI_n_maxLines[];
	extern	const	char	OI_n_maxSize[];
	extern	const	char	OI_n_maximumLength[];
	extern	const	char	OI_n_maximum[];
	extern	const	char	OI_n_menuTrigger[];
	extern	const	char	OI_n_minLabel[];
	extern	const	char	OI_n_minSize[];
	extern	const	char	OI_n_minimum[];
	extern	const	char	OI_n_mnemonicLabel[];
	extern	const	char	OI_n_mnemonicStyle[];
	extern	const	char	OI_n_mnemonics[];
	extern	const	char	OI_n_model[];
	extern	const	char	OI_n_modeless[];
	extern	const	char	OI_n_modelSpecificHints[];
	extern	const	char	OI_n_motifPushpin[];
	extern	const	char	OI_n_motionCallback[];
	extern	const	char	OI_n_move[];
	extern	const	char	OI_n_multiClickDelta[];
	extern	const	char	OI_n_nameObjects[];
	extern	const	char	OI_n_noInherit[];
	extern	const	char	OI_n_numTick[];
	extern	const	char	OI_n_objectBoxHeight[];
	extern	const	char	OI_n_objectBoxWidth[];
	extern	const	char	OI_n_objectResources[];
	extern	const	char	OI_n_object[];
	extern	const	char	OI_n_orientation[];
	extern	const	char	OI_n_override[];
	extern	const	char	OI_n_pasteAtPointer[];
	extern	const	char	OI_n_path[];
	extern	const	char	OI_n_persistent[];
	extern	const	char	OI_n_pinned[];
	extern	const	char	OI_n_pixelRange[];
	extern	const	char	OI_n_pixelsX[];
	extern	const	char	OI_n_pixelsY[];
	extern	const	char	OI_n_placement[];
	extern	const	char	OI_n_position[];
	extern	const	char	OI_n_pushpin[];
	extern	const	char	OI_n_realTimeDrag[];
	extern	const	char	OI_n_resizeByGrip[];
	extern	const	char	OI_n_resize[];
	extern	const	char	OI_n_resourceFile[];
	extern	const	char	OI_n_restrictTerm[];
	extern	const	char	OI_n_reverseVideo[];
	extern	const	char	OI_n_rightMargin[];
	extern	const	char	OI_n_rightSpace[];
	extern	const	char	OI_n_row[];
	extern	const	char	OI_n_scrollMenuTimeout[];
	extern	const	char	OI_n_selected[];
	extern	const	char	OI_n_separator[];
	extern	const	char	OI_n_shapeFile[];
	extern	const	char	OI_n_showSep[];
	extern	const	char	OI_n_showValue[];
	extern	const	char	OI_n_sizeTrack[];
	extern	const	char	OI_n_space[];
	extern	const	char	OI_n_spanUpdate[];
	extern	const	char	OI_n_spanX[];
	extern	const	char	OI_n_spanY[];
	extern	const	char	OI_n_standardColormap[];
	extern	const	char	OI_n_state[];
	extern	const	char	OI_n_subMenuType[];
	extern	const	char	OI_n_subbox[];
	extern	const	char	OI_n_subclassFilePath[];
	extern	const	char	OI_n_submenu[];
	extern	const	char	OI_n_substitute[];
	extern	const	char	OI_n_synchronous[];
	extern	const	char	OI_n_tabWidth[];
	extern	const	char	OI_n_textFile[];
	extern	const	char	OI_n_text[];
	extern	const	char	OI_n_tickLabels[];
	extern	const	char	OI_n_tickType[];
	extern	const	char	OI_n_title[];
	extern	const	char	OI_n_topBevelColor[];
	extern	const	char	OI_n_topLevel[];
	extern	const	char	OI_n_topSpace[];
	extern	const	char	OI_n_top[];
	extern	const	char	OI_n_track[];
	extern	const	char	OI_n_trackCells[];
	extern	const	char	OI_n_translations[];
	extern	const	char	OI_n_underline[];
	extern	const	char	OI_n_unitX[];
	extern	const	char	OI_n_unitY[];
	extern	const	char	OI_n_unselect[];
	extern	const	char	OI_n_userArgRqd[];
	extern	const	char	OI_n_userArg[];
	extern	const	char	OI_n_userVariable[];
	extern	const	char	OI_n_validateText[];
	extern	const	char	OI_n_value[];
	extern	const	char	OI_n_vertMove[];
	extern	const	char	OI_n_vertResize[];
	extern	const	char	OI_n_viewChars[];
	extern	const	char	OI_n_viewHeight[];
	extern	const	char	OI_n_viewLines[];
	extern	const	char	OI_n_viewSize[];
	extern	const	char	OI_n_viewWidth[];
	extern	const	char	OI_n_viewX[];
	extern	const	char	OI_n_viewY[];
	extern	const	char	OI_n_viewportFitsText[];
	extern	const	char	OI_n_width[];
	extern	const	char	OI_n_wmIgnoresPPosition[];
	extern	const	char	OI_n_wrap[];
	extern	const	char	OI_n_wrapAround[];
	extern	const	char	OI_n_wrapLimit[];
	extern	const	char	OI_n_x[];
	extern	const	char	OI_n_y[];
	extern	const	char	OI_r_Alignment[];
	extern	const	char	OI_r_Argp[];
	extern	const	char	OI_r_Atom[];
	extern	const	char	OI_r_BevelPixels[];
	extern	const	char	OI_r_BevelWidth[];
	extern	const	char	OI_r_Bool[];
	extern	const	char	OI_r_Boolean[];
	extern	const	char	OI_r_BorderWidth[];
	extern	const	char	OI_r_Children[];
	extern	const	char	OI_r_Class[];
	extern	const	char	OI_r_Color[];
	extern	const	char	OI_r_Controllers[];
	extern	const	char	OI_r_CursorShape[];
	extern	const	char	OI_r_Cursor[];
	extern	const	char	OI_r_Display[];
	extern	const	char	OI_r_Enhance[];
	extern	const	char	OI_r_FileQuark[];
	extern	const	char	OI_r_File[];
	extern	const	char	OI_r_Float[];
	extern	const	char	OI_r_FocusIndicator[];
	extern	const	char	OI_r_FocusPolicy[];
	extern	const	char	OI_r_FontStruct[];
	extern	const	char	OI_r_Font[];
	extern	const	char	OI_r_Function[];
	extern	const	char	OI_r_Geometry[];
	extern	const	char	OI_r_Height[];
	extern	const	char	OI_r_Immediate[];
	extern	const	char	OI_r_Int[];
	extern	const	char	OI_r_Language[];
	extern	const	char	OI_r_Layout[];
	extern	const	char	OI_r_Long[];
	extern	const	char	OI_r_MallocString[];
	extern	const	char	OI_r_MemberFunction[];
	extern	const	char	OI_r_MnemonicStyle[];
	extern	const	char	OI_r_Model[];
	extern	const	char	OI_r_ModifierMask[];
	extern	const	char	OI_r_OI_bevel_style[];
	extern	const	char	OI_r_OI_char_encode_type[];
	extern	const	char	OI_r_OI_display_1d_current[];
	extern	const	char	OI_r_OI_display_1d_ends[];
	extern	const	char	OI_r_OI_display_1d_ticks[];
	extern	const	char	OI_r_OI_font[];
	extern	const	char	OI_r_OI_font_bounds[];
	extern	const	char	OI_r_OI_gravity[];
	extern	const	char	OI_r_OI_lang_focus[];
	extern	const	char	OI_r_OI_number[];
	extern	const	char	OI_r_OI_pixmap[];
	extern	const	char	OI_r_OI_position[];
	extern	const	char	OI_r_OI_size_track[];
	extern	const	char	OI_r_OI_controller_visibility[];
	extern	const	char	OI_r_OI_state[];
	extern	const	char	OI_r_OI_track[];
	extern	const	char	OI_r_Object[];
	extern	const	char	OI_r_Orientation[];
	extern	const	char	OI_r_Path[];
	extern	const	char	OI_r_Pixel[];
	extern	const	char	OI_r_Pointer[];
	extern	const	char	OI_r_Position[];
	extern	const	char	OI_r_QuarkList[];
	extern	const	char	OI_r_Quark[];
	extern	const	char	OI_r_QuotedStringList[];
	extern	const	char	OI_r_QuotedString[];
	extern	const	char	OI_r_Short[];
	extern	const	char	OI_r_StandardColormap[];
	extern	const	char	OI_r_State[];
	extern	const	char	OI_r_String[];
	extern	const	char	OI_r_TranslationTable[];
	extern	const	char	OI_r_UnsignedChar[];
	extern	const	char	OI_r_Visual[];
	extern	const	char	OI_r_Width[];
	extern	const	char	OI_r_Window[];
	extern	const	char	OI_r_X[];
	extern	const	char	OI_r_Y[];

#ifdef OI_XT_RESOURCES
#define DefaultFont OI_DefaultFont

#define XtDefaultBackground OI_default_background
#define XtDefaultFont OI_default_font
#define XtDefaultForeground OI_default_foreground
#define XtDefaultLanguage OI_default_language

#define XtCAbbrCellPaintAsCell OI_c_AbbrCellPaintAsCell
#define XtCAcceleratorLabel OI_c_AcceleratorLabel
#define XtCAccelerators OI_c_Accelerators
#define XtCAlignment OI_c_Alignment
#define XtCAppHelpFile OI_c_AppHelpFile
#define XtCAppHelpTopic OI_c_AppHelpTopic
#define XtCApplication OI_c_Application
#define XtCArgp OI_c_Argp
#define XtCAugment OI_c_Augment
#define XtCAutoCntrlrVisibility OI_c_AutoCntrlrVisibility
#define XtCAutoResize OI_c_AutoResize
#define XtCBackgroundPixmap OI_c_BackgroundPixmap
#define XtCBackground OI_c_Background
#define XtCBarActiveColor OI_c_BarActiveColor
#define XtCBarInactiveColor OI_c_BarInactiveColor
#define XtCBarSize OI_c_BarSize
#define XtCBaseClass OI_c_BaseClass
#define XtCBevelStyle OI_c_BevelStyle
#define XtCBevelWidth OI_c_BevelWidth
#define XtCBitmapFilePath OI_c_BitmapFilePath
#define XtCBitmap OI_c_Bitmap
#define XtCBorderColor OI_c_BorderColor
#define XtCBorderWidth OI_c_BorderWidth
#define XtCBottomBevelColor OI_c_BottomBevelColor
#define XtCBottomSpace OI_c_BottomSpace
#define XtCCallback OI_c_Callback
#define XtCCallbacks OI_c_Callbacks
#define XtCCellOutline OI_c_CellOutline
#define XtCCharEncodeType OI_c_CharEncodeType
#define XtCChildren OI_c_Children
#define XtCClickDelta OI_c_ClickDelta
#define XtCClickSelect OI_c_ClickSelect
#define XtCClip OI_c_Clip
#define XtCCollapse OI_c_Collapse
#define XtCColumn OI_c_Column
#define XtCConfig OI_c_Config
#define XtCControllers OI_c_Controllers
#define XtCControllerVisibility OI_c_ControllerVisibility
#define XtCCursor OI_c_Cursor
#define XtCCutPaste OI_c_CutPaste
#define XtCDecimalPosition OI_c_DecimalPosition
#define XtCDefaultCell OI_c_DefaultCell
#define XtCDefaultHorizontalSpace OI_c_DefaultHorizontalSpace
#define XtCDefaultKey OI_c_DefaultKey
#define XtCDefaultLanguage OI_c_DefaultLanguage
#define XtCDefaultText OI_c_DefaultText
#define XtCDefaultVerticalSpace OI_c_DefaultVerticalSpace
#define XtCDefaults OI_c_Defaults
#define XtCDirectory OI_c_Directory
#define XtCDisplayLength OI_c_DisplayLength
#define XtCDownColor OI_c_DownColor
#define XtCDownIsBackground OI_c_DownIsBackground
#define XtCDragRightDistance OI_c_DragRightDistance
#define XtCDynamicResources OI_c_DynamicResources
#define XtCEndType OI_c_EndType
#define XtCFastDraw OI_c_FastDraw
#define XtCFile OI_c_File
#define XtCFilterLabel OI_c_FilterLabel
#define XtCFilter OI_c_Filter
#define XtCFocusFrameColor OI_c_FocusFrameColor
#define XtCFocusFramePixmap OI_c_FocusFramePixmap
#define XtCFocusFrameWidth OI_c_FocusFrameWidth
#define XtCFocusIndicator OI_c_FocusIndicator
#define XtCFocusPolicy OI_c_FocusPolicy
#define XtCFontSet OI_c_FontSet
#define XtCFont OI_c_Font
#define XtCForeground OI_c_Foreground
#define XtCFrameWidth OI_c_FrameWidth
#define XtCFunction OI_c_Function
#define XtCGeometry OI_c_Geometry
#define XtCGravity OI_c_Gravity
#define XtCHandleIndent OI_c_HandleIndent
#define XtCHeight OI_c_Height
#define XtCHelpFile OI_c_HelpFile
#define XtCHelpTopic OI_c_HelpTopic
#define XtCHelpTranslations OI_c_HelpTranslations
#define XtCHorzMove OI_c_HorzMove
#define XtCHorzResize OI_c_HorzResize
#define XtCIconFilename OI_c_IconFilename
#define XtCIconGeometry OI_c_IconGeometry
#define XtCIconX OI_c_IconX
#define XtCIconY OI_c_IconY
#define XtCIconic OI_c_Iconic
#define XtCIgnoreMouseLock OI_c_IgnoreMouseLock
#define XtCIgnoreModifierMask OI_c_IgnoreModifierMask
#define XtCInactiveCursor OI_c_InactiveCursor
#define XtCIncDecChk OI_c_IncDecChk
#define XtCIndependentLoc OI_c_IndependentLoc
#define XtCInlineEnhance OI_c_InlineEnhance
#define XtCKeyboardInput OI_c_KeyboardInput
#define XtCLabel OI_c_Label
#define XtCLanguageServerFocusinAction OI_c_LanguageServerFocusinAction
#define XtCLanguageServer OI_c_LanguageServer
#define XtCLanguage OI_c_Language
#define XtCLayout OI_c_Layout
#define XtCLeftSpace OI_c_LeftSpace
#define XtCLength OI_c_Length
#define XtCLong OI_c_Long
#define XtCLongterm OI_c_Longterm
#define XtCMainMenu OI_c_MainMenu
#define XtCMapRaised OI_c_MapRaised
#define XtCMargin OI_c_Margin
#define XtCMask OI_c_Mask
#define XtCMaxChars OI_c_MaxChars
#define XtCMaxLabel OI_c_MaxLabel
#define XtCMaxLines OI_c_MaxLines
#define XtCMaxSize OI_c_MaxSize
#define XtCMaximumLength OI_c_MaximumLength
#define XtCMaximum OI_c_Maximum
#define XtCMinLabel OI_c_MinLabel
#define XtCMinSize OI_c_MinSize
#define XtCMinimum OI_c_Minimum
#define XtCMnemonicLabel OI_c_MnemonicLabel
#define XtCMnemonicStyle OI_c_MnemonicStyle
#define XtCMnemonics OI_c_Mnemonics
#define XtCModel OI_c_Model
#define XtCModeless OI_c_Modeless
#define XtCModelSpecificHints OI_c_ModelSpecificHints
#define XtCMotifPushpin OI_c_MotifPushpin
#define XtCMotionCallback OI_c_MotionCallback
#define XtCMove OI_c_Move
#define XtCNoInherit OI_c_NoInherit
#define XtCNumTick OI_c_NumTick
#define XtCObjectBoxHeight OI_c_ObjectBoxHeight
#define XtCObjectBoxWidth OI_c_ObjectBoxWidth
#define XtCObjectResources OI_c_ObjectResources
#define XtCObject OI_c_Object
#define XtCOrientation OI_c_Orientation
#define XtCOverride OI_c_Override
#define XtCPath
#define XtCPersistent OI_c_Persistent
#define XtCPinned OI_c_Pinned
#define XtCPixelRange OI_c_PixelRange
#define XtCPixelsX OI_c_PixelsX
#define XtCPixelsY OI_c_PixelsY
#define XtCPlacement OI_c_Placement
#define XtCPushpin OI_c_Pushpin
#define XtCRealTimeDrag OI_c_RealTimeDrag
#define XtCResizeByGrip OI_c_ResizeByGrip
#define XtCResize OI_c_Resize
#define XtCResourceFile OI_c_ResourceFile
#define XtCRestrictTerm OI_c_RestrictTerm
#define XtCReverseVideo OI_c_ReverseVideo
#define XtCRightSpace OI_c_RightSpace
#define XtCRow OI_c_Row
#define XtCScrollMenuTimeout OI_c_ScrollMenuTimeout
#define XtCSelected OI_c_Selected
#define XtCSeparator OI_c_Separator
#define XtCShapeFile OI_c_ShapeFile
#define XtCShowSep OI_c_ShowSep
#define XtCShowValue OI_c_ShowValue
#define XtCSizeTrack OI_c_SizeTrack
#define XtCSpace OI_c_Space
#define XtCSpanUpdate OI_c_SpanUpdate
#define XtCSpanX OI_c_SpanX
#define XtCSpanY OI_c_SpanY
#define XtCStandardColormap OI_c_StandardColormap
#define XtCState OI_c_State
#define XtCSubMenuType OI_c_SubMenuType
#define XtCSubbox OI_c_Subbox
#define XtCSubclassFilePath OI_c_SubclassFilePath
#define XtCSubmenu OI_c_Submenu
#define XtCSubstitute OI_c_Substitute
#define XtCSynchronous OI_c_Synchronous
#define XtCTabWidth OI_c_TabWidth
#define XtCTextFile OI_c_TextFile
#define XtCText OI_c_Text
#define XtCTickLabels OI_c_TickLabels
#define XtCTickPosition OI_c_TickPosition
#define XtCTickType OI_c_TickType
#define XtCTitle OI_c_Title
#define XtCTopBevelColor OI_c_TopBevelColor
#define XtCTopLevel OI_c_TopLevel
#define XtCTopSpace OI_c_TopSpace
#define XtCTrack OI_c_Track
#define XtCTranslations OI_c_Translations
#define XtCUnderline OI_c_Underline
#define XtCUnitX OI_c_UnitX
#define XtCUnitY OI_c_UnitY
#define XtCUnselect OI_c_Unselect
#define XtCUserArgRqd OI_c_UserArgRqd
#define XtCUserArg OI_c_UserArg
#define XtCUserVariable OI_c_UserVariable
#define XtCValidateText OI_c_ValidateText
#define XtCValue OI_c_Value
#define XtCVertMove OI_c_VertMove
#define XtCVertResize OI_c_VertResize
#define XtCViewChars OI_c_ViewChars
#define XtCViewHeight OI_c_ViewHeight
#define XtCViewLines OI_c_ViewLines
#define XtCViewSize OI_c_ViewSize
#define XtCViewWidth OI_c_ViewWidth
#define XtCViewX OI_c_ViewX
#define XtCViewY OI_c_ViewY
#define XtCViewportFitsText OI_c_ViewportFitsText
#define XtCWidth OI_c_Width
#define XtCWmIgnoresPPosition OI_c_WmIgnoresPPosition
#define XtCWrapAround OI_c_WrapAround
#define XtCWrapLimit OI_c_WrapLimit
#define XtCX OI_c_X
#define XtCY OI_c_Y
#define XtNabbrCellPaintAsCell OI_n_abbrCellPaintAsCell
#define XtNacceleratorLabel OI_n_acceleratorLabel
#define XtNaccelerators OI_n_accelerators
#define XtNalignment OI_n_alignment
#define XtNappHelpFile OI_n_appHelpFile
#define XtNappHelpTopic OI_n_appHelpTopic
#define XtNapplication OI_n_application
#define XtNargp OI_n_argp
#define XtNaugment OI_n_augment
#define XtNautoCntrlrVisibility OI_n_autoCntrlrVisibility
#define XtNautoResize OI_n_autoResize
#define XtNbackgroundPixmap OI_n_backgroundPixmap
#define XtNbackground OI_n_background
#define XtNbarActiveColor OI_n_barActiveColor
#define XtNbarInactiveColor OI_n_barInactiveColor
#define XtNbarSize OI_n_barSize
#define XtNbaseClass OI_n_baseClass
#define XtNbevelStyle OI_n_bevelStyle
#define XtNbevelWidth OI_n_bevelWidth
#define XtNbitmapFilePath OI_n_bitmapFilePath
#define XtNbitmap OI_n_bitmap
#define XtNborderColor OI_n_borderColor
#define XtNborderWidth OI_n_borderWidth
#define XtNborder OI_n_border
#define XtNbottomBevelColor OI_n_bottomBevelColor
#define XtNbottomSpace OI_n_bottomSpace
#define XtNcallback OI_n_callback
#define XtNcallbacks OI_n_callbacks
#define XtNcbAnimate OI_n_cbAnimate
#define XtNcbCellAction OI_n_cbCellAction
#define XtNcbClick OI_n_cbClick
#define XtNcbCmdEntryChk OI_n_cbCmdEntryChk
#define XtNcbCtlr1d OI_n_cbCtlr1d
#define XtNcbCtlr2d OI_n_cbCtlr2d
#define XtNcbDecCheck OI_n_cbDecCheck
#define XtNcbDeiconify OI_n_cbDeiconify
#define XtNcbDestroy OI_n_cbDestroy
#define XtNcbDragPreview OI_n_cbDragPreview
#define XtNcbDropPreview OI_n_cbDropPreview
#define XtNcbDrop OI_n_cbDrop
#define XtNcbEfCharChg OI_n_cbEfCharChg
#define XtNcbEfCharChk OI_n_cbEfCharChk
#define XtNcbEfEntryChk OI_n_cbEfEntryChk
#define XtNcbExpose OI_n_cbExpose
#define XtNcbFocusin OI_n_cbFocusin
#define XtNcbFocusout OI_n_cbFocusout
#define XtNcbGripMoved OI_n_cbGripMoved
#define XtNcbGripStart OI_n_cbGripStart
#define XtNcbGripStop OI_n_cbGripStop
#define XtNcbIconify OI_n_cbIconify
#define XtNcbIncCheck OI_n_cbIncCheck
#define XtNcbKeyHelp OI_n_cbKeyHelp
#define XtNcbMenuEnd OI_n_cbMenuEnd
#define XtNcbMtCharChg OI_n_cbMtCharChg
#define XtNcbMtCharChk OI_n_cbMtCharChk
#define XtNcbMtEntryChk OI_n_cbMtEntryChk
#define XtNcbOIDestroy OI_n_cbOIDestroy
#define XtNcbOIResize OI_n_cbOIResize
#define XtNcbOITrackReloc OI_n_cbOITrackReloc
#define XtNcbPanPaint OI_n_cbPanPaint
#define XtNcbPin OI_n_cbPin
#define XtNcbPrePopup OI_n_cbPrePopup
#define XtNcbProtocolDelete OI_n_cbProtocolDelete
#define XtNcbResize OI_n_cbResize
#define XtNcbSaveYourself OI_n_cbSaveYourself
#define XtNcbShutdown OI_n_cbShutdown
#define XtNcbTermAction OI_n_cbTermAction
#define XtNcbUnpin OI_n_cbUnpin
#define XtNcellOutline OI_n_cellOutline
#define XtNcharEncodeType OI_n_charEncodeType
#define XtNchildren OI_n_children
#define XtNclickDelta OI_n_clickDelta
#define XtNclickSelect OI_n_clickSelect
#define XtNclip OI_n_clip
#define XtNcollapse OI_n_collapse
#define XtNcolormap OI_n_colormap
#define XtNcolumn OI_n_column
#define XtNconfig OI_n_config
#define XtNcontrollers OI_n_controllers
#define XtNcursor OI_n_cursor
#define XtNcutPaste OI_n_cutPaste
#define XtNdata OI_n_data
#define XtNdecimalPosition OI_n_decimalPosition
#define XtNdefaultCell OI_n_defaultCell
#define XtNdefaultHorizontalSpace OI_n_defaultHorizontalSpace
#define XtNdefaultKey OI_n_defaultKey
#define XtNdefaultLanguage OI_n_defaultLanguage
#define XtNdefaultText OI_n_defaultText
#define XtNdefaultVerticalSpace OI_n_defaultVerticalSpace
#define XtNdefaults OI_n_defaults
#define XtNdirectory OI_n_directory
#define XtNdisplayLength OI_n_displayLength
#define XtNdownColor OI_n_downColor
#define XtNdownIsBackground OI_n_downIsBackground
#define XtNdragRightDistance OI_n_dragRightDistance
#define XtNdynamicResources OI_n_dynamicResources
#define XtNendType OI_n_endType
#define XtNfastDraw OI_n_fastDraw
#define XtNfile OI_n_file
#define XtNfilterLabel OI_n_filterLabel
#define XtNfilter OI_n_filter
#define XtNfocusFrameColor OI_n_focusFrameColor
#define XtNfocusFramePixmap OI_n_focusFramePixmap
#define XtNfocusFrameWidth OI_n_focusFrameWidth
#define XtNfocusIndicator OI_n_focusIndicator
#define XtNfocusPolicy OI_n_focusPolicy
#define XtNfontSet OI_n_fontSet
#define XtNfont OI_n_font
#define XtNforeground OI_n_foreground
#define XtNframeWidth OI_n_frameWidth
#define XtNfunction OI_n_function
#define XtNgeometry OI_n_geometry
#define XtNgravity OI_n_gravity
#define XtNhandleIndent OI_n_handleIndent
#define XtNheight OI_n_height
#define XtNhelpFile OI_n_helpFile
#define XtNhelpTopic OI_n_helpTopic
#define XtNhelpTranslations OI_n_helpTranslations
#define XtNhorzMove OI_n_horzMove
#define XtNhorzResize OI_n_horzResize
#define XtNiconFilename OI_n_iconFilename
#define XtNiconGeometry OI_n_iconGeometry
#define XtNiconX OI_n_iconX
#define XtNiconY OI_n_iconY
#define XtNiconic OI_n_iconic
#define XtNignoreMouseLock OI_n_ignoreMouseLock
#define XtNignoreModifierMask OI_n_ignoreModifierMask
#define XtNinactiveCursor OI_n_inactiveCursor
#define XtNincDecChk OI_n_incDecChk
#define XtNindependentLoc OI_n_independentLoc
#define XtNinlineEnhance OI_n_inlineEnhance
#define XtNkeyboardInput OI_n_keyboardInput
#define XtNlabel OI_n_label
#define XtNlanguageServerFocusinAction OI_n_languageServerFocusinAction
#define XtNlanguageServer OI_n_languageServer
#define XtNlanguage OI_n_language
#define XtNlayout OI_n_layout
#define XtNleftSpace OI_n_leftSpace
#define XtNlength OI_n_length
#define XtNlong OI_n_long
#define XtNlongterm OI_n_longterm
#define XtNmainMenu OI_n_mainMenu
#define XtNmapRaised OI_n_mapRaised
#define XtNmappedWhenManaged OI_n_mappedWhenManaged
#define XtNmarginHeight OI_n_marginHeight
#define XtNmarginWidth OI_n_marginWidth
#define XtNmask OI_n_mask
#define XtNmaxChars OI_n_maxChars
#define XtNmaxLabel OI_n_maxLabel
#define XtNmaxLines OI_n_maxLines
#define XtNmaxSize OI_n_maxSize
#define XtNmaximumLength OI_n_maximumLength
#define XtNmaximum OI_n_maximum
#define XtNmenuTrigger OI_n_menuTrigger
#define XtNminLabel OI_n_minLabel
#define XtNminSize OI_n_minSize
#define XtNminimum OI_n_minimum
#define XtNmnemonicLabel OI_n_mnemonicLabel
#define XtNmnemonicStyle OI_n_mnemonicStyle
#define XtNmnemonics OI_n_mnemonics
#define XtNmodel OI_n_model
#define XtNmodeless OI_n_modeless
#define XtNmodelSpecificHints OI_n_modelSpecificHints
#define XtNmotifPushpin OI_n_motifPushpin
#define XtNmotionCallback OI_n_motionCallback
#define XtNmove OI_n_move
#define XtNmultiClickDelta OI_n_multiClickDelta
#define XtNnoInherit OI_n_noInherit
#define XtNnumTick OI_n_numTick
#define XtNobjectBoxHeight OI_n_objectBoxHeight
#define XtNobjectBoxWidth OI_n_objectBoxWidth
#define XtNobjectResources OI_n_objectResources
#define XtNobject OI_n_object
#define XtNorientation OI_n_orientation
#define XtNoverride OI_n_override
#define XtNpath OI_n_path
#define XtNpersistent OI_n_persistent
#define XtNpinned OI_n_pinned
#define XtNpixelRange OI_n_pixelRange
#define XtNpixelsX OI_n_pixelsX
#define XtNpixelsY OI_n_pixelsY
#define XtNplacement OI_n_placement
#define XtNpushpin OI_n_pushpin
#define XtNrealTimeDrag OI_n_realTimeDrag
#define XtNresizeByGrip OI_n_resizeByGrip
#define XtNresize OI_n_resize
#define XtNresourceFile OI_n_resourceFile
#define XtNrestrictTerm OI_n_restrictTerm
#define XtNreverseVideo OI_n_reverseVideo
#define XtNrightSpace OI_n_rightSpace
#define XtNrow OI_n_row
#define XtNscrollMenuTimeout OI_n_scrollMenuTimeout
#define XtNselected OI_n_selected
#define XtNseparator OI_n_separator
#define XtNshapeFile OI_n_shapeFile
#define XtNshowSep OI_n_showSep
#define XtNshowValue OI_n_showValue
#define XtNsizeTrack OI_n_sizeTrack
#define XtNcontrollerVisibility OI_n_controllerVisibility
#define XtNspace OI_n_space
#define XtNspanUpdate OI_n_spanUpdate
#define XtNspanX OI_n_spanX
#define XtNspanY OI_n_spanY
#define XtNstandardColormap OI_n_standardColormap
#define XtNstate OI_n_state
#define XtNsubMenuType OI_n_subMenuType
#define XtNsubbox OI_n_subbox
#define XtNsubclassFilePath OI_n_subclassFilePath
#define XtNsubmenu OI_n_submenu
#define XtNsubstitute OI_n_substitute
#define XtNsynchronous OI_n_synchronous
#define XtNtabWidth OI_n_tabWidth
#define XtNtextFile OI_n_textFile
#define XtNtext OI_n_text
#define XtNtickLabels OI_n_tickLabels
#define XtNtickType OI_n_tickType
#define XtNtitle OI_n_title
#define XtNtopBevelColor OI_n_topBevelColor
#define XtNtopLevel OI_n_topLevel
#define XtNtopSpace OI_n_topSpace
#define XtNtop OI_n_top
#define XtNtrack OI_n_track
#define XtNtranslations OI_n_translations
#define XtNunderline OI_n_underline
#define XtNunitX OI_n_unitX
#define XtNunitY OI_n_unitY
#define XtNunselect OI_n_unselect
#define XtNuserArgRqd OI_n_userArgRqd
#define XtNuserArg OI_n_userArg
#define XtNuserVariable OI_n_userVariable
#define XtNvalidateText OI_n_validateText
#define XtNvalue OI_n_value
#define XtNvertMove OI_n_vertMove
#define XtNvertResize OI_n_vertResize
#define XtNviewChars OI_n_viewChars
#define XtNviewHeight OI_n_viewHeight
#define XtNviewLines OI_n_viewLines
#define XtNviewSize OI_n_viewSize
#define XtNviewWidth OI_n_viewWidth
#define XtNviewX OI_n_viewX
#define XtNviewY OI_n_viewY
#define XtNviewportFitsText OI_n_viewportFitsText
#define XtNwidth OI_n_width
#define XtNwmIgnoresPPosition OI_n_wmIgnoresPPosition
#define XtNwrapAround OI_n_wrapAround
#define XtNwrapLimit OI_n_wrapLimit
#define XtNx OI_n_x
#define XtNy OI_n_y
#define XtRAlignment OI_r_Alignment
#define XtRArgp OI_r_Argp
#define XtRAtom OI_r_Atom
#define XtRBevelPixels OI_r_BevelPixels
#define XtRBevelWidth OI_r_BevelWidth
#define XtRBool OI_r_Bool
#define XtRBoolean OI_r_Boolean
#define XtRBorderWidth OI_r_BorderWidth
#define XtRChildren OI_r_Children
#define XtRClass OI_r_Class
#define XtRColor OI_r_Color
#define XtRControllers OI_r_Controllers
#define XtRCursorShape OI_r_CursorShape
#define XtRCursor OI_r_Cursor
#define XtRDisplay OI_r_Display
#define XtREnhance OI_r_Enhance
#define XtRFileQuark OI_r_FileQuark
#define XtRFile OI_r_File
#define XtRFloat OI_r_Float
#define XtRFocusIndicator OI_r_FocusIndicator
#define XtRFocusPolicy OI_r_FocusPolicy
#define XtRFontStruct OI_r_FontStruct
#define XtRFont OI_r_Font
#define XtRFunction OI_r_Function
#define XtRGeometry OI_r_Geometry
#define XtRHeight OI_r_Height
#define XtRImmediate OI_r_Immediate
#define XtRInt OI_r_Int
#define XtRLanguage OI_r_Language
#define XtRLayout OI_r_Layout
#define XtRLong OI_r_Long
#define XtRMallocString OI_r_MallocString
#define XtRMemberFunction OI_r_MemberFunction
#define XtRMnemonicStyle OI_r_MnemonicStyle
#define XtRModel OI_r_Model
#define XtRModifierMask OI_r_ModifierMask
#define XtROI_bevel_style OI_r_OI_bevel_style
#define XtROI_char_encode_type OI_r_OI_char_encode_type
#define XtROI_display_1d_current OI_r_OI_display_1d_current
#define XtROI_display_1d_ends OI_r_OI_display_1d_ends
#define XtROI_display_1d_ticks OI_r_OI_display_1d_ticks
#define XtROI_font OI_r_OI_font
#define XtROI_font_bounds OI_r_OI_font_bounds
#define XtROI_gravity OI_r_OI_gravity
#define XtROI_lang_focus OI_r_OI_lang_focus
#define XtROI_number OI_r_OI_number
#define XtROI_pixmap OI_r_OI_pixmap
#define XtROI_position OI_r_OI_position
#define XtROI_size_track OI_r_OI_size_track
#define XtROI_controller_visibility OI_r_OI_controller_visibility
#define XtROI_state OI_r_OI_state
#define XtROI_track OI_r_OI_track
#define XtRObject OI_r_Object
#define XtROrientation OI_r_Orientation
#define XtRPath OI_r_Path
#define XtRPixel OI_r_Pixel
#define XtRPointer OI_r_Pointer
#define XtRPosition OI_r_Position
#define XtRQuarkList OI_r_QuarkList
#define XtRQuark OI_r_Quark
#define XtRQuotedStringList OI_r_QuotedStringList
#define XtRQuotedString OI_r_QuotedString
#define XtRShort OI_r_Short
#define XtRStandardColormap OI_r_StandardColormap
#define XtRState OI_r_State
#define XtRString OI_r_String
#define XtRTranslationTable OI_r_TranslationTable
#define XtRUnsignedChar OI_r_UnsignedChar
#define XtRVisual OI_r_Visual
#define XtRWidth OI_r_Width
#define XtRWindow OI_r_Window
#define XtRX OI_r_X
#define XtRY OI_r_Y

#endif /* OI_XT_RESOURCES */

#endif /* OI_resname_H */
