/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	deldly.H
 *
 *	Functions(s):
 *
 *	Description:
 *		Definitions for delete_delayed object
 *
 *	RCSid = "$Id: deldly.H,v 4.1.1.1 1993/06/01 22:53:55 mth Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_deldly_H
#define OI_deldly_H

	class	OI_del_delay	{
			OI_d_tech	**queuep ;	/* ptr to que of objects to delete; NULL slot => open */
			OI_bool		*allqp ;	/* OI_YES => corresponding object in queuep needs delete_all() */
			OI_number	len ;		/* length of queue */
			OI_d_tech	**qp ;		/* ptr to next entry in queue */
			OI_bool		*aqp ;		/* ptr to next entry in allqp */
	 public:
					OI_del_delay(OI_number) ;
					~OI_del_delay() ;
			void		add(class OI_d_tech*,OI_bool) ;
			OI_number	length()			{ return(len); }
			void		set_length(OI_number) ;
	} ;

 /* ptr to delete delayed queue */
	extern	class	OI_del_delay	*OI_del_delayp ;	/* ptr to delayed delete object queue */

#endif /* OI_deldly_H */
