/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	root.H
 *
 *	Functions(s):
 *
 *	Description:
 *		OI_root and OI_orphanage class header
 *
 *	RCSid = "$Id: root.H,v 4.1.1.1 1993/06/01 22:56:27 mth Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_root_H
#define OI_root_H

#ifndef OI_wdtech_H
#include <OI/wdtech.H>
#endif /* OI_wdtech_H */

	class OI_root : public OI_w_d_tech {
	 public:
		static	OI_class	*clsp ;
	 private:
			OI_stat		create() ;
	 public:
					OI_root(const char*) ;
					~OI_root() ;
	} ;

	class OI_orphanage : public OI_w_d_tech {
	 public:
		static	OI_class	*clsp ;
	 private:
			OI_stat		create() ;
	 public:
					OI_orphanage() ;
					~OI_orphanage() ;
	} ;

#endif /* OI_root_H */
