/* RCSid = "$Id: Xutil.H,v 4.0.1.1 1993/06/01 22:52:15 mth Exp $" */
#include <X11/Xutil.h>

#ifdef ultrix
/* Prototypes moved from DEC's Xlib.H because they depend on types from Xutil.h */

/*
 *	ICCCM stuff
 */
extern int XWMGeometry(
#if NeedFunctionPrototypes
    Display*            /* display */,
    int                 /* screen_number */,
    const char*         /* user_geometry */,
    const char*         /* default_geometry */,
    unsigned int        /* border_width */,
    XSizeHints*         /* hints */,
    int*                /* x_return */,
    int*                /* y_return */,
    int*                /* width_return */,
    int*                /* height_return */,
    int*                /* gravity_return */
#endif
);

extern Status XGetWMSizeHints(
#if NeedFunctionPrototypes
    Display*            /* display */,
    Window              /* w */,
    XSizeHints*         /* hints_return */,
    long*               /* supplied_return */,
    Atom                /* property */
#endif
);

extern Status XGetWMNormalHints(
#if NeedFunctionPrototypes
    Display*            /* display */,
    Window              /* w */,
    XSizeHints*         /* hints_return */,
    long*               /* supplied_return */
#endif
);

extern Status XGetRGBColormaps(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XStandardColormap** /* stdcmap_return */,
    int*		/* count_return */,
    Atom		/* property */
#endif
);

extern Status XGetTextProperty(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* window */,
    XTextProperty*	/* text_prop_return */,
    Atom		/* property */
#endif
);

extern Status XGetWMName(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* text_prop_return */
#endif
);

extern Status XGetWMIconName(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* text_prop_return */
#endif
);

extern Status XGetWMClientMachine(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* text_prop_return */
#endif
);

extern void XSetWMProperties(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* window_name */,
    XTextProperty*	/* icon_name */,
    char**		/* argv */,
    int			/* argc */,
    XSizeHints*		/* normal_hints */,
    XWMHints*		/* wm_hints */,
    XClassHint*		/* class_hints */
#endif
);

extern void XSetWMSizeHints(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XSizeHints*		/* hints */,
    Atom		/* property */
#endif
);

extern void XSetWMNormalHints(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XSizeHints*		/* hints */
#endif
);

extern void XSetRGBColormaps(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XStandardColormap*	/* stdcmaps */,
    int			/* count */,
    Atom		/* property */
#endif
);

extern void XSetTextProperty(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* text_prop */,
    Atom		/* property */
#endif
);

extern void XSetWMName(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* text_prop */
#endif
);

extern void XSetWMIconName(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* text_prop */
#endif
);

extern void XSetWMClientMachine(
#if NeedFunctionPrototypes
    Display*		/* display */,
    Window		/* w */,
    XTextProperty*	/* text_prop */
#endif
);

extern Status XStringListToTextProperty(
#if NeedFunctionPrototypes
    char**		/* list */,
    int			/* count */,
    XTextProperty*	/* text_prop_return */
#endif
);

extern Status XTextPropertyToStringList(
#if NeedFunctionPrototypes
    XTextProperty*	/* text_prop */,
    char***		/* list_return */,
    int*		/* count_return */
#endif
);

extern Status XGetSizeHints(
#if NeedFunctionPrototypes
    Display*            /* display */,
    Window              /* w */,
    XSizeHints*         /* hints_return */,
    Atom                /* property */
#endif
);

#endif
