/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	atname.H
 *
 *	Functions(s):
 *
 *	Description:
 *		Names for internal parts of composite OI objects.
 *		All internal names begin with an at sign (@).
 *		The declarations are sorted case independently.
 *
 *	RCSid = "$Id: atname.H,v 4.4.1.2 1993/08/04 03:44:25 toml Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_atname_H
#define OI_atname_H

		extern	const	char	OI_at_bar_spacer[];
		extern	const	char	OI_at_bottom_scroll_bar[];
		extern	const	char	OI_at_box_interior[];
		extern	const	char	OI_at_bt_hdr[];
		extern	const	char	OI_at_buttons[];
		extern	const	char	OI_at_clone[];
		extern	const	char	OI_at_command_cell[];
		extern	const	char	OI_at_connection_help[];
		extern	const	char	OI_at_default_topic_cell[];
		extern	const	char	OI_at_default_topics[];
		extern	const	char	OI_at_directory_entry_field[];
		extern	const	char	OI_at_directory_scroll_menu[];
		extern	const	char	OI_at_discard[];
		extern	const	char	OI_at_discard_menu[];
		extern	const	char	OI_at_done[];
		extern	const	char	OI_at_entry_field[];
		extern	const	char	OI_at_error_dialog_box[];
		extern	const	char	OI_at_filter[];
		extern	const	char	OI_at_filter_entry_field[];
		extern	const	char	OI_at_filter_static_text[];
		extern	const	char	OI_at_glyph[];
		extern	const	char	OI_at_grip[];
		extern	const	char	OI_at_header[];
		extern	const	char	OI_at_help[];
		extern	const	char	OI_at_help_button[];
		extern	const	char	OI_at_help_container[];
		extern	const	char	OI_at_help_menu[];
		extern	const	char	OI_at_help_subtree[];
		extern	const	char	OI_at_hyper_help[];
		extern	const	char	OI_at_icon[];
		extern	const	char	OI_at_in_doc[];
		extern	const	char	OI_at_in_top[];
		extern	const	char	OI_at_interior[];
		extern	const	char	OI_at_left_scroll_bar[];
		extern	const	char	OI_at_lf_hdr[];
		extern	const	char	OI_at_magnifier[];
		extern	const	char	OI_at_main_menu[];
		extern	const	char	OI_at_menu[];
		extern	const	char	OI_at_menu_button[];
		extern	const	char	OI_at_menu_cell[];
		extern	const	char	OI_at_misc[];
		extern	const	char	OI_at_miscellaneous[];
		extern	const	char	OI_at_multi_text[];
		extern	const	char	OI_at_navigation_menu[];
		extern	const	char	OI_at_object_box[];
		extern	const	char	OI_at_ok[];
		extern	const	char	OI_at_on_context[];
		extern	const	char	OI_at_orphanage[];
		extern	const	char	OI_at_pane[];
		extern	const	char	OI_at_panner[];
		extern	const	char	OI_at_pin_menu[];
		extern	const	char	OI_at_pin_unpin[];
		extern	const	char	OI_at_reg_exp_cell[];
		extern	const	char	OI_at_reg_exp_menu[];
		extern	const	char	OI_at_ret_prv[];
		extern	const	char	OI_at_right_scroll_bar[];
		extern	const	char	OI_at_rt_hdr[];
		extern	const	char	OI_at_root_window[];
		extern	const	char	OI_at_scrollbar_menu[];
		extern	const	char	OI_at_scrollbox[];
		extern	const	char	OI_at_search[];
		extern	const	char	OI_at_search_box[];
		extern	const	char	OI_at_select_menu[];
		extern	const	char	OI_at_sep_maj[];
		extern	const	char	OI_at_sep_min[];
		extern	const	char	OI_at_separator[];
		extern	const	char	OI_at_srch[];
		extern	const	char	OI_at_state[];
		extern	const	char	OI_at_static_text[];
		extern	const	char	OI_at_subclass[];
		extern	const	char	OI_at_template[];
		extern	const	char	OI_at_text[];
		extern	const	char	OI_at_title[];
		extern	const	char	OI_at_title_spacer[];
		extern	const	char	OI_at_top[];
		extern	const	char	OI_at_top_scroll_bar[];
		extern	const	char	OI_at_tp_hdr[];
		extern	const	char	OI_at_viewport[];
		extern	const	char	OI_at_virtual_root[];
		extern	const	char	OI_at_visit_tree[];
		extern	const	char	OI_at_visits[];
		extern	const	char	OI_at_pushpin[];

#endif /* OI_atname_H */
