/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	str.H
 *
 *	Functions(s):
 *
 *	Description:
 *		Misc strings
 *
 *	RCSid = "$Id: str.H,v 4.12.1.2 1993/08/04 03:44:25 toml Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_str_H
#define OI_str_H

		extern	const	char	OI_str_iconGeometry[];
		extern	const	char	OI_str_wm_hints[];
		extern	const	char	OI_str_backgroundPixmap[];
		extern	const	char	OI_str_acceleratorsRecord[];
		extern	const	char	OI_str_mnemonicsRecord[];
		extern	const	char	OI_str_language[];
		extern	const	char	OI_str_colormap[];
		extern	const	char	OI_str_accelerators[];
		extern	const	char	OI_str_mnemonics[];
		extern	const	char	OI_str_drag[];
		extern	const	char	OI_str_drop[];
		extern	const	char	OI_str_selectionHandler[];
		extern	const	char	OI_str_placement[];
		extern	const	char	OI_str_sizeConstraints[];
		extern	const	char	OI_str_wm_normal_hints[] ;
		extern	const	char	OI_str_wm_protocols[] ;
		extern	const	char	OI_str_wm_class[] ;
		extern	const	char	OI_str_geometry[];
		extern	const	char	OI_str_ol_decor_add[];
		extern	const	char	OI_str_ol_decor_del[];
		extern	const	char	OI_str_p_ol_decor_del[];	// private
		extern	const	char	OI_str_p_ol_decor_add[];	// private
		extern	const	char	OI_str_ol_win_attr[];
		extern	const	char	OI_str_motif_wm_hints[];
		extern	const	char	OI_str_p_motif_wm_hints[];	// private
		extern	const	char	OI_str_mouse_x[];
		extern	const	char	OI_str_drop_sites[];
		extern	const	char	OI_str_drop_targets[];
		extern	const	char	OI_str_selection_targets[];
		extern	const	char	OI_str_work_freeze_win[];
		extern	const	char	OI_str_res_children[];

		extern	const	char	OI_str_WM_STATE[] ;

		extern	const	char	OI_str_MOTIF_WM_INFO[] ;
		extern	const	char	OI_str_MOTIF_WM_HINTS[] ;

		extern	const	char	OI_str_SUN_WM_PROTOCOLS[] ;
		extern	const	char	OI_str_SUN_OL_WIN_ATTR_5[] ;

		extern	const	char	OI_str_OI_TIME[] ;
		extern	const	char	OI_str_OL_DECOR_ADD[] ;
		extern	const	char	OI_str_OL_DECOR_DEL[] ;
		extern	const	char	OI_str_OL_DECOR_HEADER[] ;
		extern	const	char	OI_str_OL_DECOR_PIN[] ;
		extern	const	char	OI_str_OL_DECOR_RESIZE[] ;
		extern	const	char	OI_str_OL_DECOR_CLOSE[] ;
		extern	const	char	OI_str_OL_DFLT_BTN[] ;
		extern	const	char	OI_str_OL_NONE[] ;
		extern	const	char	OI_str_OL_PIN_IN[] ;
		extern	const	char	OI_str_OL_PIN_OUT[] ;
		extern	const	char	OI_str_OL_PIN_STATE[] ;
		extern	const	char	OI_str_OL_WIN_ATTR[] ;
		extern	const	char	OI_str_OL_WIN_BUSY[] ;
		extern	const	char	OI_str_OL_WT_BASE[] ;
		extern	const	char	OI_str_OL_WT_CMD[] ;
		extern	const	char	OI_str_OL_WT_HELP[] ;
		extern	const	char	OI_str_OL_WT_NOTICE[] ;
		extern	const	char	OI_str_OL_WT_OTHER[] ;
		extern	const	char	OI_str_OL_MENU_FULL[] ;
		extern	const	char	OI_str_OL_MENU_LIMITED[] ;

		extern	const	char	OI_str_SWM_VERSION[] ;

		extern	const	char	OI_str_TARGETS[];
		extern	const	char	OI_str_TIMESTAMP[];
		extern	const	char	OI_str_FILE_NAME[];
		extern	const	char	OI_str_MULTIPLE[];
		extern	const	char	OI_str_STRING[];
		extern	const	char	OI_str_LENGTH[];
		extern	const	char	OI_str_HOST_NAME[];
		extern	const	char	OI_str_ATOM[];
		extern	const	char	OI_str_INTEGER[];
		extern	const	char	OI_str_DELETE[];

		extern	const	char	OI_DND_SUN_DRAGDROP_INTEREST[];
		extern	const	char	OI_DND_SUN_DRAGDROP_TRIGGER[];
		extern	const	char	OI_DND_SUN_DRAGDROP_ACK[];
		extern	const	char	OI_DND_SUN_ENUMERATION_COUNT[];
		extern	const	char	OI_DND_SUN_ENUMERATION_ITEM[];
		extern	const	char	OI_DND_SUN_FILE_HOST_NAME[];
		extern	const	char	OI_DND_SUN_AVAILABLE_TYPES[];
		extern	const	char	OI_DND_SUN_DRAGDROP_DONE[];
		extern	const	char	OI_DND_SUN_DRAGDROP_PREVIEW[];
		extern	const	char	OI_DND_SUN_DRAGDROP_DSDM[];
		extern	const	char	OI_DND_SUN_DRAGDROP_SITE_RECTS[];

		extern	const	char	OI_DND_MOTIF_DRAG_RECEIVER_INFO[];
		extern	const	char	OI_DND_MOTIF_DRAG_INITIATOR_INFO[];
		extern	const	char	OI_DND_MOTIF_DRAG_AND_DROP_MESSAGE[];
		extern	const	char	OI_DND_MOTIF_TRANSFER_SUCCESS[];
		extern	const	char	OI_DND_MOTIF_DRAG_WINDOW[];
		extern	const	char	OI_DND_MOTIF_DRAG_TARGETS[];

		extern	const	char	OI_DND_ENUMERATION_COUNT[];
		extern	const	char	OI_DND_ENUMERATION_ITEM[];
		extern	const	char	OI_DND_ENUMERATION_END[];
		extern	const	char	OI_DND_TARGETS[];
		extern	const	char	OI_DND_MULTIPLE[];
		extern	const	char	OI_DND_FILE_NAME[];
		extern	const	char	OI_DND_STRING[];
		extern	const	char	OI_DND_FILE_HOST_NAME[];
		extern	const	char	OI_DND_HOST_NAME[];
		extern	const	char	OI_DND_AVAILABLE_TYPES[];
		extern	const	char	OI_DND_LENGTH[];
		extern	const	char	OI_DND_DONE[];
		extern	const	char	OI_DND_TRANSIENT[];
		extern	const	char	OI_DND_SUN_SITE_RECTS[];
		extern	const	char	OI_DND_DELETE[];
		extern	const	char	OI_DND_TIMESTAMP[];
		extern	const	char	OI_DND_TRANSFER_SUCCESS[];

		extern	const	char	OI_SELECTION_DATA[];
		
#endif /* OI_str_H */
