/*
 *	RCSid = "$Id"
 */

/* This is a copy of string.h for use on the UTS system,
 * which does not currently supply it.
 */
/*ident	"@(#)ctrans:incl-master/proto-headers/string.h	1.5" */

#ifndef __STRING_H
#define __STRING_H

#ifndef __MEMORY_H
#include <memory.h>
#endif

/*	@(#)string.h 1.4 88/02/07 SMI; from S5R2 1.2	*/

extern "C" {
	char *strcpy(char*, const char*);
	char *strncpy(char*, const char*, int);
	char *strcat(char*, const char*);
	char *strncat(char*, const char*, int);
	int strcmp(const char*, const char*);
	int strncmp(const char*, const char*, int);
	int strlen(const char*);
	char *strchr(const char*, int);
	char *strpbrk(const char*, const char*);
	char *strrchr(const char*, int);
	char *strtok(char*, const char*);
	int strspn(const char*, const char*);
	int strcspn(const char*, const char*);
	char *index(const char*, char);
	char *rindex(const char*, char);
	char *strdup(const char*);
}

#endif
