/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	exchkmnu.H
 *
 *	Functions(s):
 *
 *	Description:
 *		OI_excl_check_menu class header
 *
 *	RCSid = "$Id: exchkmnu.H,v 4.2.1.1 1993/06/01 22:54:18 mth Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_exchkmnu_H
#define OI_exchkmnu_H

#ifndef OI_exmnu_H
#include <OI/exmnu.H>
#endif /* OI_exmnu_H */

	// class forward reference declaration
	class OI_mdl_excl_check_menu;

	class OI_excl_check_menu : public OI_excl_menu {
			OI_mdl_excl_check_menu	*mdl_dep ;
	 public:
		static	OI_class		*clsp ;			/* ptr to class record */
	 private:
			void			construct(OI_number, OI_menu_cell**) ;
	 protected:
						OI_excl_check_menu(OI_class *, const char *, OI_number, OI_menu_cell **,
							OI_orient =OI_VERTICAL, const char * =NULL) ;
						OI_excl_check_menu(OI_class *, const char *, OI_number, OI_cell_spec *,
							OI_orient =OI_VERTICAL, const char * =NULL) ;
		virtual	OI_number		bullet_space_x() ;
		virtual	OI_number		bullet_size_x() ;
		virtual	void			new_model(OI_model_type) ;
         public:
		//-------------------------------------------------------------------------------------------------		
		//		
		//	RESTRICTED MEMBER FUNCTIONS
		//		Functions in this section are not documented in the programmer's manual and are
		//		restricted to internal use by the toolkit.
		//		Functions not contained in user documentation are subject to change without notice
		//		and should never be used by toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//
		//-------------------------------------------------------------------------------------------------		
						OI_excl_check_menu(const char *, OI_number, OI_menu_cell **, OI_orient or=OI_VERTICAL,
							const char *ttl=NULL) ;
						OI_excl_check_menu(const char *, OI_number, OI_cell_spec *, OI_orient or=OI_VERTICAL,
							const char *ttl=NULL) ;
						~OI_excl_check_menu() ;
		virtual	void			cell_paint(OI_menu_cell *cp) ;
		static	OI_bool			init(class OI_connection*) ;
		static	OI_excl_check_menu	*make_minimal(const char*,OI_minimal_type) ;

		//-------------------------------------------------------------------------------------------------		
		//		
		//	OVERRIDES OF DOCUMENTED PARENT MEMBER FUNCTIONS	
		//		Functions in this section are documented in the programmer's manual under the
		//		parent(s) of this object and are also available to the toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//
		//-------------------------------------------------------------------------------------------------		

		//-------------------------------------------------------------------------------------------------		
		//		
		//	DOCUMENTED MEMBER FUNCTIONS	
		//		Functions below this point are documented in the programmer's manual under this
		//		object and are available to the toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//
		//-------------------------------------------------------------------------------------------------		

	} ;

//-------------------------------------------------------------------------------------------------		
//		
//	RESTRICTED FUNCTIONS
//		Functions in this section are not documented in the programmer's manual and are
//		restricted to internal use by the toolkit.
//		Functions not contained in user documentation are subject to change without notice
//		and should never be used by toolkit users.
//		Please maintain alphabetical order to facilitate releases and documentation.
//
//-------------------------------------------------------------------------------------------------		

//-------------------------------------------------------------------------------------------------		
//		
//	DOCUMENTED FUNCTIONS	
//		Functions below this point are documented in the programmer's manual under this
//		object and are available to the toolkit users.
//		Please maintain alphabetical order to facilitate releases and documentation.
//
//-------------------------------------------------------------------------------------------------		

#endif /* OI_exchkmnu_H */
