/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	hdrbox.H
 *
 *	Functions(s):
 *
 *	Description:
 *		Object Interface OI_heading_box class header
 *
 *	RCSid = "$Id: hdrbox.H,v 4.1.1.1 1993/06/01 22:54:45 mth Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_hdrbox_H
#define OI_hdrbox_H

#ifndef OI_box_H
#include <OI/box.H>
#endif /* OI_box_H */

	class OI_heading_box : public OI_box {
	 private:
	 protected:
	 public:
		//-------------------------------------------------------------------------------------------------		
		//	RESTRICTED MEMBER FUNCTIONS
		//		Functions in this section are not documented in the programmer's manual and are
		//		restricted to internal use by the toolkit.
		//		Functions not contained in user documentation are subject to change without notice
		//		and should never be used by toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//-------------------------------------------------------------------------------------------------		
					OI_heading_box(const char*) ;
					~OI_heading_box() ;
			OI_box		*object_box() ;

		//-------------------------------------------------------------------------------------------------		
		//	OVERRIDES OF DOCUMENTED PARENT MEMBER FUNCTIONS	
		//		Functions in this section are documented in the programmer's manual under the
		//		parent(s) of this object and are also available to the toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//-------------------------------------------------------------------------------------------------		
		//-------------------------------------------------------------------------------------------------		
		//	DOCUMENTED MEMBER FUNCTIONS	
		//		Functions below this point are documented in the programmer's manual under this
		//		object and are available to the toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//-------------------------------------------------------------------------------------------------		
			void		position(OI_number,OI_number) ;
	} ;

//-------------------------------------------------------------------------------------------------		
//	RESTRICTED FUNCTIONS
//		Functions in this section are not documented in the programmer's manual and are
//		restricted to internal use by the toolkit.
//		Functions not contained in user documentation are subject to change without notice
//		and should never be used by toolkit users.
//		Please maintain alphabetical order to facilitate releases and documentation.
//-------------------------------------------------------------------------------------------------		

//-------------------------------------------------------------------------------------------------		
//	DOCUMENTED FUNCTIONS	
//		Functions below this point are documented in the programmer's manual under this
//		object and are available to the toolkit users.
//		Please maintain alphabetical order to facilitate releases and documentation.
//-------------------------------------------------------------------------------------------------		
	OI_heading_box  *oi_create_heading_box(const char*) ;

#endif /* OI_hdrbox_H */
