/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	warndlg.H
 *
 *	Functions(s):
 *
 *	Description:
 *		OI_warn_dialog_box class header
 *
 *	RCSid = "$Id: warndlg.H,v 4.2.1.1 1993/06/01 22:57:15 mth Exp $"
 *
 ****************************************************************************
 */

#ifndef	OI_warndlg_H
#define	OI_warndlg_H

#ifndef	OI_msdlg_H
#include <OI/msdlg.H>
#endif /* OI_msdlg_H */

	// class forward reference declarations
	class OI_mdl_warn_dialog_box;

class OI_warn_dialog_box : public OI_ms_dialog_box {
			OI_mdl_warn_dialog_box	*mdl_dep;	/* pointer to model dependent warn dlg box. */
	 public:
		static	OI_class		*clsp ;		/* ptr to class record */
	 private:
			void			construct() ;
	 protected:
						OI_warn_dialog_box(OI_class*, const char*, const char* =NULL, OI_number =-1,
							OI_menu_cell** =NULL, OI_glyph* =NULL) ;
						OI_warn_dialog_box(OI_class*, const char*, const char*, OI_number, OI_cell_spec*,
							OI_glyph*) ;
		virtual	void			new_model(OI_model_type) ;
	 public:
		//-------------------------------------------------------------------------------------------------		
		//		
		//	RESTRICTED MEMBER FUNCTIONS
		//		Functions in this section are not documented in the programmer's manual and are
		//		restricted to internal use by the toolkit.
		//		Functions not contained in user documentation are subject to change without notice
		//		and should never be used by toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//
		//-------------------------------------------------------------------------------------------------		
						OI_warn_dialog_box(const char*, const char* =NULL, OI_number =-1, OI_menu_cell** =NULL,
							OI_glyph* =NULL);
						OI_warn_dialog_box(const char*, const char*, OI_number, OI_cell_spec*, OI_glyph*);
						~OI_warn_dialog_box() ;
		static	OI_bool			init(OI_connection*) ;
		static	OI_warn_dialog_box	*make_minimal(const char*,OI_minimal_type) ;

		//-------------------------------------------------------------------------------------------------		
		//		
		//	OVERRIDES OF DOCUMENTED PARENT MEMBER FUNCTIONS	
		//		Functions in this section are documented in the programmer's manual under the
		//		parent(s) of this object and are also available to the toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//
		//-------------------------------------------------------------------------------------------------		

		//-------------------------------------------------------------------------------------------------		
		//		
		//	DOCUMENTED MEMBER FUNCTIONS	
		//		Functions below this point are documented in the programmer's manual under this
		//		object and are available to the toolkit users.
		//		Please maintain alphabetical order to facilitate releases and documentation.
		//
		//-------------------------------------------------------------------------------------------------		

};

//-------------------------------------------------------------------------------------------------		
//		
//	RESTRICTED FUNCTIONS
//		Functions in this section are not documented in the programmer's manual and are
//		restricted to internal use by the toolkit.
//		Functions not contained in user documentation are subject to change without notice
//		and should never be used by toolkit users.
//		Please maintain alphabetical order to facilitate releases and documentation.
//
//-------------------------------------------------------------------------------------------------		

//-------------------------------------------------------------------------------------------------		
//		
//	OBSOLETED FUNCTIONS	
//		Functions below this point were once documented in the programmer's manual under
//		this object, but are now obsolete and will be removed in a future release.
//		Please maintain alphabetical order to facilitate releases and documentation.
//
//-------------------------------------------------------------------------------------------------		
	OI_warn_dialog_box	*oi_warn_dialog_box(const char*, const char* =NULL, OI_number =-1, OI_menu_cell** =NULL, OI_glyph* =NULL);

//-------------------------------------------------------------------------------------------------		
//		
//	DOCUMENTED FUNCTIONS	
//		Functions below this point are documented in the programmer's manual under this
//		object and are available to the toolkit users.
//		Please maintain alphabetical order to facilitate releases and documentation.
//
//-------------------------------------------------------------------------------------------------		
	OI_warn_dialog_box	*oi_create_warn_dialog_box(const char*, const char* =NULL, OI_number =-1, OI_cell_spec* =NULL,
					OI_glyph* =NULL);
	OI_warn_dialog_box	*oi_create_warn_dialog_box(const char*, const char*, OI_number, OI_menu_cell**, OI_glyph* =NULL);

#endif /* OI_warndlg_H */
