/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	evlpstk.H
 *
 *	Functions(s):
 *
 *	Description:
 *		Class definition for main event loop stack
 *
 *	RCSid = "$Id: evlpstk.H,v 4.0.1.1 1993/06/01 22:54:16 mth Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_evlpstk_H
#define OI_evlpstk_H

	class OI_event_loop_stack {			/* stack of process flags for event loops */
			OI_bool		**stkp ;	/* ptr to stack of flags for outstanding OI_wait_done calls */
			OI_bool		**topp ;	/* ptr to top of stack of flags for outstanding OI_wait_done calls (current ele) */
			OI_number	mx ;		/* max size of wait stack */
		 public:
					OI_event_loop_stack() ;
			void		push(OI_bool*) ;	/* push new element onto stack */
			void		pop() ;			/* pop topmost entry from stack */
			OI_bool		validate() ;		/* chk if all entries still active */
	} ;

#endif /* OI_evlpstk_H */
