#ifndef _Xct_H
#define _Xct_H

/*	RCSid = "$Id: Xct.H,v 4.0.1.1 1993/06/01 22:57:27 mth Exp $" */

// The X11R4 Xmu/Xct.h header is NOT prototyped, so borrow a trick
// from the C++ versions of the Unix system headers
#define	XctCreate	______XctCreate
#define	XctNextItem	______XctNextItem
#define	XctFree		______XctFree
#define	XctReset	______XctReset

#if defined(ultrix) || defined(hpux)
#include <OI/Xct.h>
#else
#include <X11/Xmu/Xct.h>
#endif

#undef	XctCreate
#undef	XctNextItem
#undef	XctFree	
#undef	XctReset

/* these are the external routines */
extern	XctData		XctCreate(XctString,int,XctFlags);
extern	XctResult	XctNextItem(XctData);
extern	void		XctFree(XctData);
extern	void		XctReset(XctData);

#endif
