/*
 ***************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA. All rights reserved.
 *
 ***************************************************************************
 *
 *	File:	filarr.H
 *
 *	Function(s):
 *
 *	Description:
 *		file_array class.  Useful to maintain an array of
 *		sorted files, then do usefult things with them (like
 *		build a menu).
 *
 *	RCSid = "$Id: filarr.H,v 4.1.1.1 1993/06/01 22:54:26 mth Exp $"
 *
 ***************************************************************************
 */

#ifndef OI_filarr_H
#define OI_filarr_H

class OI_file_array {
	int		count;
	char		*namep;
	OI_file_array	*left;
	OI_file_array	*right;
	void		make_menu_cell (OI_menu_cell **, int &);
 public:
			OI_file_array () ;
			OI_file_array (char *) ;
			~OI_file_array () ;
	void		add_name (char *name);
	void		set_menu (OI_scroll_menu *);
};

#endif /* OI_filarr_H */
