/*
 ******************************************************************************
 *
 *	ParcPlace Systems, Inc.
 *	Copyright (c) 1990, ParcPlace Systems, Inc. USA   All rights reserved.
 *
 ****************************************************************************
 *
 *	File:	subclass.H
 *
 *	Functions(s):
 *
 *	Description:
 *		Definitions normally needed by and OI_d_tech subclass
 *
 *	RCSid = "$Id: subclass.H,v 1.1.1.1 1993/06/01 22:57:00 mth Exp $"
 *
 ****************************************************************************
 */

#ifndef OI_subclass_H
#define OI_subclass_H

#include <X11/X.h>
#include <X11/Xatom.h>

#ifndef OI_d_tech_H
#include <OI/d_tech.H>
#endif /* OI_d_tech_H */

#ifndef OI_cb_inf_H
#include <OI/cb_inf.H>
#endif /* OI_cb_inf_H */

#ifndef OI_class_H
#include <OI/class.H>
#endif /* OI_class_H */

#ifndef OI_connect_H
#include <OI/connect.H>
#endif /* OI_connect_H */

#ifndef OI_cvt_H
#include <OI/cvt.H>
#endif /* OI_cvt_H */

#ifndef OI_dispatch_H
#include <OI/dispatch.H>
#endif /* OI_dispatch_H */

#ifndef OI_font_H
#include <OI/font.H>
#endif /* OI_font_H */

#ifndef OI_plystr_H
#include <OI/plystr.H>
#endif /* OI_plystr_H */

#ifndef OI_resname_H
#include <OI/resname.H>
#endif /* OI_resname_H */

#ifndef OI_str_H
#include <OI/str.H>
#endif /* OI_str_H */

#ifndef OI_tab_H
#include <OI/tab.H>
#endif /* OI_tab_H */

#ifndef OI_tm_H
#include <OI/tm.H>
#endif /* OI_tm_H */

#endif /* OI_subclass_H */
