#ifndef OI_VERSION_H
#define OI_VERSION_H
/* There are three components to an OI version number:
 * The major release number, digits 5 and 4,
 * the minor release number, digits 3 and 2,
 * and the tape number, digits 1 and 0.
 * This version has the following components:
 *     MAJOR = 4
 *     MINOR = 00
 *     TAPE  = 08
 */
/* OI_VERSION is a concatination of MAJOR and MINOR */
#define OI_VERSION 400
/* OI_EXACT_VERSION is a concatination of MAJOR, MINOR, and TAPE */
#define OI_EXACT_VERSION 40008
extern char *OI_version;
#endif /* OI_VERSION_H */
